<?php

namespace Drupal\proc\Form\Encrypt;

use Drupal\Component\Utility\Crypt;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\proc\Entity\Proc;

/**
 * Open a modal form to encrypt an uploaded file.
 *
 * @todo this is still a WIP version of a less monolithic upload.
 */
class EncryptFileModalForm extends AbstractEncryptionForm {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'proc_encrypt_modal_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    // Check users
    // Filter those without a proc key.
    // --> Rest show public keys
    // --> If no valid ones --> Warning message.

    // Get query string:
    $query = $this->getRequest()->query->all();
    $form['proc-file'] = [
      '#type' => 'file',
      '#description' => $this->t('Select a file for encryption.'),
      'data' => $query,
    ];

    // If stand-alone mode is false, then this encryption
    // is triggered by a field widget and the dialog has to
    // be closed by ajax submit.
    $form['#attached']['drupalSettings'] = [
      'proc' => [
        'proc_standalone_mode' => FALSE,
      ],
    ];

    $form['submit-proc'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#ajax' => [
        // Close dialog using ajax submit:
        'callback' => '::submitFormAjax',
        'event' => 'click',
      ],
    ];

    return $form;
  }

  /**
   * Submit handler for the ajax submit.
   */
  public function submitFormAjax(array &$form, FormStateInterface $form_state) {
    // Close the dialog:
    $response = new AjaxResponse();
    // Selector for the dialog close button.
    // @todo Isn't there a close command for the modal?
    $selector = '.ui-dialog-titlebar-close';
    $method = 'click';
    $arguments = [];
    $response->addCommand(new InvokeCommand($selector, $method, $arguments));

    // Get the latest proc ID unless this is triggered by a field widget:
    // @todo, this is a pretty wild assumption, that the last proc would be the one you want.
    $query = \Drupal::entityQuery('proc');
    $query->accessCheck(TRUE);
    $query->sort('id', 'DESC');
    $query->range(0, 1);
    $entity_ids = $query->execute();
    $proc_id = array_shift($entity_ids);

    $proc_object_label = Proc::load($proc_id)
      ->get('label')
      ->getValue()[0]['value'];

    // @todo, add more terse way of passing this target.
    $selector = '#edit-field-encrypted-attachment-0-display';
    $file_name = $proc_object_label . ' (' . $proc_id . ')';
    $response->addCommand(new InvokeCommand($selector, 'val', [$file_name]));

    return $response;
  }

  /**
   * Implements form validation.
   *
   * The validateForm method is the default method called to validate input on
   * a form.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // @todo Add validation for the syntax of armored cipher text.
  }

  /**
   * Implements a form submit handler.
   *
   * The submitForm method is the default method called for any submit elements.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    global $base_url;

    $recipients_set_ids = $form_state->get('storage');

    // @todo, what is all this?
    $destination = FALSE;
    $current_url = \Drupal::request()->headers->get('referer');
    $parse_result = UrlHelper::parse($current_url);

    if (isset($parse_result['query']['destination'])) {
      $destination = $parse_result['query']['destination'];
    }

    $cipher = ['cipher' => $form_state->getValue('cipher_text')];
    $meta = [
      'source_file_name' => $form_state->getValue('source_file_name'),
      'source_file_size' => $form_state->getValue('source_file_size'),
      'source_file_type' => $form_state->getValue('source_file_type'),
      'source_file_last_change' => $form_state->getValue('source_file_last_change'),
      'browser_fingerprint' => $form_state->getValue('browser_fingerprint'),
      'generation_timestamp' => $form_state->getValue('generation_timestamp'),
      'generation_timespan' => $form_state->getValue('generation_timespan'),
      'signed' => $form_state->getValue('signed'),
    ];

    $proc = Proc::create();

    $recipient_users = [];
    foreach ($recipients_set_ids as $recipient_id) {
      $recipient_users[] = ['target_id' => $recipient_id];
    }

    $json_content = $cipher['cipher'];
    $fragment = Crypt::hashBase64(Crypt::randomBytesBase64(32));
    $json_filename = $fragment . '.json';

    $config = \Drupal::config('proc.settings');

    // @todo Move this into a helper.
    $enable_stream_wrapper = $config->get('proc-enable-stream-wrapper');
    $stream_wrapper = $config->get('proc-stream-wrapper');
    $block_size = $config->get('proc-file-block-size');
    $blocks_split_enabled = $config->get('proc-enable-block-size');

    $file_id = FALSE;
    if ($enable_stream_wrapper === 1 && !empty($stream_wrapper)) {
      $json_dest = $stream_wrapper;
      if (!is_dir($json_dest)) {
        \Drupal::service('file_system')->mkdir($json_dest, NULL, TRUE);
      }

      if ($json_content) {
        if ($blocks_split_enabled && !empty($block_size)) {
          // $content_lines_number = substr_count($json_content, "\n");
          $lines = explode("\n", $json_content);
          $content_lines_number = count($lines);

          $lines_size_ratio = $content_lines_number / $block_size;
          $blocks = intval($lines_size_ratio);

          $remaining = $content_lines_number % $block_size;
          if ($remaining > 0) {
            $blocks++;
          }
          $blocks_index = 0;

          $blocks_lines = [];
          $content_line_index = 0;
          while ($blocks_index < $blocks) {
            $line_in_block_index = 0;
            while ($line_in_block_index < $block_size) {
              $blocks_lines[$blocks_index][] = $lines[$content_line_index];
              $content_line_index++;
              $line_in_block_index++;
            }
            $blocks_index++;
          }

          $blocks_texts = [];
          foreach ($blocks_lines as $block_index => $block_lines) {
            foreach ($block_lines as $block_line) {
              $blocks_texts[$block_index] = $blocks_texts[$block_index] . "\n" . $block_line;
            }
          }

          $json_fids = [];
          foreach ($blocks_texts as $block_text) {
            $fragment = Crypt::hashBase64(Crypt::randomBytesBase64(32));
            $json_filename = $fragment . '.json';

            $jsonFid = \Drupal::service('file.repository')
              ->writeData(
                $block_text,
                "$json_dest/$json_filename",
                FileSystemInterface::EXISTS_REPLACE
              );
            if ($jsonFid->id()) {
              $json_fids[] = $jsonFid->id();
            }
          }
        }
        else {
          $jsonFid = \Drupal::service('file.repository')
            ->writeData(
              $json_content,
              "$json_dest/$json_filename",
              FileSystemInterface::EXISTS_REPLACE
            );

          if ($jsonFid->id()) {
            $file_id = $jsonFid->id();
          }
        }
      }
    }

    if ($file_id) {
      $cipher = ['cipher_fid' => $file_id];
    }
    if (!empty($json_fids)) {
      $cipher = ['cipher_fid' => $json_fids];
    }

    $saved = $proc->set('armored', $cipher)
      ->set('meta', $meta)
      ->set('langcode', 'en')
      ->set('label', $meta['source_file_name'])
      ->set('type', 'cipher')
      ->set('field_recipients_set', $recipient_users)
      ->save();

    // @todo add some sort of clean message here.
  }

}
