<?php

namespace Drupal\proc;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Access controller for the proc entity.
 */
class ProcAccessControlHandler extends EntityAccessControlHandler implements EntityHandlerInterface {

  const OPERATION_DECRYPT = 'decrypt';

  /**
   * The keychain service.
   *
   * @var \Drupal\proc\ProcKeychainInterface
   *   The keychain service to check or users have keyrings added.
   */
  protected ProcKeychainInterface $keychain;

  /**
   * Basic constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type this access controller is for.
   * @param \Drupal\proc\ProcKeychainInterface $keychain
   *   The keychain service to check connected keyrings.
   */
  public function __construct(EntityTypeInterface $entity_type, ProcKeychainInterface $keychain) {
    parent::__construct($entity_type);
    $this->keychain = $keychain;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static($entity_type, $container->get('proc.keychain'));
  }

  /**
   * {@inheritdoc}
   *<
   * Link the activities to the permissions. checkAccess() is called with the
   * $operation as defined in the routing.yml file.
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    // Check the admin_permission as defined in your @ContentEntityType
    // annotation.
    $admin_permission = $this->entityType->getAdminPermission();
    if ($account->hasPermission($admin_permission)) {
      return AccessResult::allowed();
    }
    switch ($operation) {
      case 'view':
        return AccessResult::allowedIfHasPermission($account, 'view proc entity');

      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit proc entity');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete proc entity');
    }
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   *
   * Separate from the checkAccess because the entity does not yet exist. It
   * will be created during the 'add' process.
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    // We block creation of cyphers if you do not have a keyring yet. Since it's a requirement.
    // @todo this can be further stabilized when the keyring and cipher are separate entities since they don't share the same use.
    // @todo add support for cypher strength.
    if (!$this->keychain->hasKeyring($account->id())) {
      return AccessResult::forbidden('A keyring is required to encrypt protected content')->addCacheTags(['proc:' . $account->id() . ':keychain']);
    }

    // Check the admin_permission as defined in your @ContentEntityType
    // annotation.
    $admin_permission = $this->entityType->getAdminPermission();
    if ($account->hasPermission($admin_permission)) {
      return AccessResult::allowed()->addCacheTags(['proc:' . $account->id() . ':keychain']);
    }

    return AccessResult::allowedIfHasPermission($account, 'add proc entity')->addCacheTags(['proc:' . $account->id() . ':keychain']);
  }

}
